import os, sys, importlib.util


def _load_module(module_name: str, path: str):

    spec = importlib.util.spec_from_file_location(module_name, path)
    if spec and spec.loader:
        mod = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(mod)
        return mod
    return None


def _load_rust_pip_or_dev(_rust_lib_name: str = '_osintxpress', module_dev_path: str = None):
    
    # Wheel load (pip install)
    try:
        from . import _osintxpress
        return _osintxpress
    except ImportError:
        pass
    
    if os.name == 'nt':
        expected_file = f'{_rust_lib_name}.dll' 
    elif sys.platform == 'darwin':
        expected_file = f'lib{_rust_lib_name}.dylib'
    else:
        expected_file = f'lib{_rust_lib_name}.so'

    py_dir = __file__.replace('\\', '/').rsplit('/', 1)[0]
    root_dir = py_dir.rsplit('/', 1)[0]
    dev_path = module_dev_path or f"{root_dir}/target/release/{expected_file}"
    
    # Search local directory (for maturin develop)
    for file in os.listdir(py_dir):
        if (file.startswith(f'lib{_rust_lib_name}') or file.startswith(_rust_lib_name)) and \
           file.endswith(('.so', '.pyd', '.dylib', '.dll')):
            mod = _load_module(_rust_lib_name, f'{py_dir}/{file}')
            if mod: return mod
    
    # Maturin build in CI
    if os.path.exists(dev_path):
        mod = _load_module(_rust_lib_name, dev_path)
        if mod: return mod

    raise ImportError(f'Could not find Rust binary. Tried local dir and {dev_path}')

_rust_lib = _load_rust_pip_or_dev()
globals().update({k: v for k, v in vars(_rust_lib).items() if not k.startswith('__')})


def supported_sources() -> list[str]:
    
    sources = [k for k in dir(SourceAdapter) if not k.startswith('_')]
    return sources



import sys, time, json, logging
import unittest
import polars as pl 

sys.path.append(__file__.replace('\\', '/').rsplit('/', 2)[0])

# Notice we now import the unified SourceAdapter instead of RestAdapter/WsAdapter
from osintxpress import OsintEngine, MockServer, SourceAdapter

logging.basicConfig(level=logging.DEBUG, format='%(levelname)s: %(message)s')

class TestOsintEngineIntegration(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        print('\n--- Starting Rust Mock Server ---')
        cls.mock_server = MockServer(host='127.0.0.1', port=0)
        
        cls.mock_server.add_rest_route(
            method='GET',
            path='/api/acled/read',
            status_code=200,
            json_payload=json.dumps({'data': [{'event_id_cnty': 'ISR123', 'fatalities': '0'}]})
        )
        
        cls.mock_server.add_rest_route(
            method='GET',
            path='/opensky/states/all',
            status_code=200,
            json_payload=json.dumps({
                "time": 1710760000, 
                "states": [
                    ["4b1814", "SWR123  ", "Switzerland", 1710760000, 1710760000, 8.54, 47.45]
                ]
            })
        )

        cls.mock_server.add_rest_route(
            method='GET',
            path='/gdelt/v2/geo',
            status_code=200,
            json_payload=json.dumps({
                "type": "FeatureCollection",
                "features": [
                    {
                        "type": "Feature",
                        "properties": {"name": "Protest", "url": "http://news.com/1"},
                        "geometry": {"type": "Point", "coordinates": [35.21, 31.76]}
                    }
                ]
            })
        )

        # Reuters Mock - Raw XML string
        cls.mock_server.add_rest_route(
            method='GET',
            path='/rss/reuters',
            status_code=200,
            raw_payload='<rss><channel><item><title>Test News</title><link>https://reuters.com/123</link></item></channel></rss>'
        )

        cls.mock_server.add_ws_route(
            path='/ws/aisstream',
            messages=[json.dumps({
                'Message': {
                    'PositionReport': {
                        'UserID': 211123456, 
                        'Sog': 12.5
                    }
                }
            })]
        )
        
        cls.mock_server.start()

    @classmethod
    def tearDownClass(cls):
        print('\n--- Stopping Rust Mock Server ---')
        cls.mock_server.stop()
        del cls.mock_server


    def test_worldmonitor_backends(self):

        engine = OsintEngine(worker_threads=2)
        
        # Using our new unified `add_source` with `source_type="rest"`
        engine.add_source(
            name='acled_conflict',
            url=f'{self.mock_server.http_url}/api/acled/read',
            source_type='rest',
            adapter=SourceAdapter.ACLED, 
            poll_interval_sec=1
        )
        
        engine.add_source(
            name='gdelt_events',
            url=f'{self.mock_server.http_url}/gdelt/v2/geo',
            source_type='rest',
            adapter=SourceAdapter.GDELT_GEOJSON,
            poll_interval_sec=1
        )
        
        engine.add_source(
            name='opensky_flights',
            url=f'{self.mock_server.http_url}/opensky/states/all',
            source_type='rest',
            adapter=SourceAdapter.OPENSKY,
            poll_interval_sec=1
        )
        
        engine.add_source(
            name='reuters_news',
            url=f'{self.mock_server.http_url}/rss/reuters',
            source_type='rest',
            adapter=SourceAdapter.REUTERS,
            poll_interval_sec=1
        )
        
        engine.add_source(
            name='ais_maritime',
            url=f'{self.mock_server.ws_url}/ws/aisstream',
            source_type='ws',
            adapter=SourceAdapter.AIS_STREAM
        )
        
        engine.start_all()
        time.sleep(1.5) 
        data = engine.poll()
        engine.stop_all()
        
        # Clean up the engine instance
        del engine
        
        # Ensure all data streams returned payloads
        assert 'acled_conflict' in data
        assert 'gdelt_events' in data
        assert 'opensky_flights' in data
        assert 'reuters_news' in data
        assert 'ais_maritime' in data
        
        assert self.mock_server.get_request_count('/api/acled/read') >= 1

        # Check ACLED Arrow Schema
        acled_df = pl.from_arrow(data['acled_conflict'])
        assert len(acled_df) >= 1
        assert acled_df['event_id_cnty'][0] == 'ISR123'

        # Check AIS Arrow Schema
        ais_df = pl.from_arrow(data['ais_maritime'])
        assert len(ais_df) >= 1
        assert 'mmsi' in ais_df.columns
        assert ais_df['mmsi'][0] == 211123456
        assert ais_df['speed'][0] == 12.5

        # Check OpenSky Arrow Schema
        opensky_df = pl.from_arrow(data["opensky_flights"])
        assert len(opensky_df) >= 1
        assert "icao24" in opensky_df.columns
        assert opensky_df["icao24"][0] == "4b1814"
        assert opensky_df["callsign"][0] == "SWR123" # Rust trims the whitespace
        assert opensky_df["latitude"][0] == 47.45

        # Check GDELT Arrow Schema
        gdelt_df = pl.from_arrow(data["gdelt_events"])
        assert len(gdelt_df) >= 1
        assert "longitude" in gdelt_df.columns
        assert gdelt_df["name"][0] == "Protest"
        assert gdelt_df["longitude"][0] == 35.21

        # Check RSS Arrow Schema
        reuters_df = pl.from_arrow(data["reuters_news"])
        assert len(reuters_df) >= 1
        assert "title" in reuters_df.columns
        assert reuters_df["title"][0] == "Test News"
        assert reuters_df["link"][0] == "https://reuters.com/123"


if __name__ == '__main__':

    unittest.main(verbosity=2)


import json

import pyarrow as pa, polars as pl, geoarrow.pyarrow as ga, panel as pn

import lonboard
from lonboard.basemap import MaplibreBasemap, CartoStyle

from osintxpress import OsintEngine, SourceAdapter


engine = OsintEngine(worker_threads=2)
engine.add_source(
    name="flights",
    source_type='rest',
    adapter=SourceAdapter.OPENSKY,
    poll_interval_sec=10 
)
engine.start_all()

# Start an empty Lonboard WebGL map
interactive_map = lonboard.Map(
    layers=[], 
    basemap=MaplibreBasemap(style=CartoStyle.DarkMatter), 
    view_state={"longitude": 34.7, "latitude": 31.5, "zoom": 5, "pitch": 45}
)

stats = pn.pane.Markdown("### Awaiting signal...", styles={"color": "#39FF14"})


def update_dashboard():

    data = engine.poll()
    if "flights" in data and len(data["flights"]) > 0:
    
        df = pl.from_arrow(data["flights"]).drop_nulls(subset=["longitude", "latitude"])
        pa_table = df.to_arrow()
        
        # GeoArrow Extension Column 
        geometry_col = ga.make_point(pa_table["longitude"], pa_table["latitude"])
        pa_table = pa_table.append_column("geometry", geometry_col)
        
        # Stream the GeoArrow binary to the browser GPU
        new_layer = lonboard.ScatterplotLayer(
            table=pa_table,
            get_fill_color=[57, 255, 20, 200], # Fancy glowy Green
            get_radius=4000,
            radius_min_pixels=2
        )
        
        # Overwrite the map layers
        interactive_map.layers = [new_layer]
        stats.object = f"### Tracking **{len(df)}** aircraft"

# Poll every 5 seconds
pn.state.add_periodic_callback(update_dashboard, period=5000)

# Render flights view
pn.Column(
    stats,
    pn.pane.IPyWidget(interactive_map, sizing_mode="stretch_both", min_height=700),
    styles={"background": "#121212", "padding": "20px"}
).servable()
